﻿//============================================================================
//
//  PA-S500制御
//
//  Ver.1.10 Date 2012/02/08
//
//   このソースにはラッパー関数の使用例が記述されています。
//
//============================================================================
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Runtime.InteropServices;

namespace Sample
{
    public partial class Pas500s : Form
    {
        private Pas500w Pasw;

        public Pas500s()
        {
            InitializeComponent();
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            short wLogSlot;         // ﾃﾞﾊﾞｲｽの論理ｽﾛｯﾄ番号を格納
            Pas500w.Resource Ri;    // ﾘｿｰｽ情報
            int dwSwVal = 0;        // ﾛｰﾀﾘｰｽｲｯﾁの値
            int dwDirections = 0;   // ﾃﾞｨﾚｸｼｮﾝ
            byte cbDat;
            bool bResult;           // 処理結果
            string szBuf;
            int i;

            // 生成
            Pasw = new Pas500w();
            Ri = new Pas500w.Resource();
            Ri.dwMemBase = new int[Pas500w.MAX_MEM];
            Ri.dwMemLength = new int[Pas500w.MAX_MEM];
            Ri.dwMemAttrib = new int[Pas500w.MAX_MEM];
            Ri.dwIOPortBase = new int[Pas500w.MAX_IO];
            Ri.dwIOPortLength = new int[Pas500w.MAX_IO];
            Ri.dwIRQRegisters = new int[Pas500w.MAX_IRQ];
            Ri.dwIRQAttrib = new int[Pas500w.MAX_IRQ];
            Ri.dwDMALst = new int[Pas500w.MAX_DMA];
            Ri.dwDMAAttrib = new int[Pas500w.MAX_DMA];
            Ri.dwReserved1 = new int[Pas500w.MAX_RSV];

            // 初期化
            Ri.dwNumMemWindows = 0;
            for (i = 0; i < Pas500w.MAX_MEM; i++)
            {
                Ri.dwMemBase[i] = 0;
                Ri.dwMemLength[i] = 0;
                Ri.dwMemAttrib[i] = 0;
            }
            Ri.dwNumIOPorts = 0;
            for (i = 0; i < Pas500w.MAX_IO; i++)
            {
                Ri.dwIOPortBase[i] = 0;
                Ri.dwIOPortLength[i] = 0;
            }
            Ri.dwNumIRQs = 0;
            for (i = 0; i < Pas500w.MAX_IRQ; i++)
            {
                Ri.dwIRQRegisters[i] = 0;
                Ri.dwIRQAttrib[i] = 0;
            }
            Ri.dwNumDMAs = 0;
            for (i = 0; i < Pas500w.MAX_DMA; i++)
            {
                Ri.dwDMALst[i] = 0;
                Ri.dwDMAAttrib[i] = 0;
            }
            for (i = 0; i < Pas500w.MAX_RSV; i++)
            {
                Ri.dwReserved1[i] = 0;
            }

            //-------------------------------------------------
            // PA-S500の使用を宣言
            //-------------------------------------------------
            wLogSlot = Pas500w.SLOT_AUTO;           // 未使用のﾃﾞﾊﾞｲｽを探します。
            bResult = Pasw.Create(ref wLogSlot);    // 成功した場合、その時のﾃﾞﾊﾞｲｽの論理ｽﾛｯﾄ番号が格納されます。
            if (bResult == false)                   // ﾃﾞﾊﾞｲｽが見つかりません。
            {
                ErrorMessage(Pas500w.SLOT_AUTO);
                this.Close();
                return;
            }
            szBuf = string.Concat("PA-S500 Created", Environment.NewLine, Environment.NewLine, string.Format("LogSlot: {0}", wLogSlot));
            ResultMessage(szBuf);
            //-------------------------------------------------
            // ﾘｿｰｽ情報とﾛｰﾀﾘｰｽｲｯﾁの値を取得
            //-------------------------------------------------
            IntPtr pri = Marshal.AllocHGlobal(Marshal.SizeOf(Ri));  // アンマネージメモリ確保
            Marshal.StructureToPtr(Ri, pri, false);                 // マーシャリング(マネージメモリ->アンマネージメモリ)
            bResult = Pasw.GetResource(wLogSlot, pri);
            Ri = (Pas500w.Resource)(Marshal.PtrToStructure(pri, typeof(Pas500w.Resource))); // マーシャリング(アンマネージメモリ->マネージメモリ)
            Marshal.FreeHGlobal(pri);                               // メモリを解放
            if (bResult == false)
            {
                ErrorMessage(wLogSlot);
                this.Close();
                return;
            }

            if (Pasw.GetSwitchValue(wLogSlot, ref dwSwVal) == false)
            {
                ErrorMessage(wLogSlot);
                this.Close();
                return;
            }
            szBuf = string.Concat("Board Name: PA-S500", Environment.NewLine, string.Format("IO Address  : {0:x4}h - {1:x4}h", Ri.dwIOPortBase[0], Ri.dwIOPortBase[0] + Ri.dwIOPortLength[0] - 1), Environment.NewLine, string.Format("Switch        : {0:x}h", dwSwVal));
            ResultMessage(szBuf);
            //-------------------------------------------------
            // Port A のﾃﾞｨﾚｸｼｮﾝを出力に設定する
            //-------------------------------------------------
            if (Pasw.GetDirections(wLogSlot, ref dwDirections) == false)
            {
                ErrorMessage(wLogSlot);
                this.Close();
                return;
            }
            dwDirections = ~Pas500w.DIR_A_INPUT & dwDirections;   // A Port のﾋﾞｯﾄを出力に変更します。
            if (Pasw.SetDirections(wLogSlot, dwDirections) == false)
            {
                ErrorMessage(wLogSlot);
                this.Close();
                return;
            }
            ResultMessage("Port A のディレクションを出力に変更");
            //-------------------------------------------------
            // Port A に任意のデータを出力
            //-------------------------------------------------
            cbDat = 0xAA;
            if (Pasw.OutPort(wLogSlot, Pas500w.PORT_A, cbDat) == false)
            {
                ErrorMessage(wLogSlot);
                this.Close();
                return;
            }
            szBuf = "Port A へデータを出力" + "\n" + "Port A Out Data: &H" + Convert.ToString(cbDat, 16).ToUpper();
            ResultMessage(szBuf);
            //-------------------------------------------------
            // Port A のﾃﾞｨﾚｸｼｮﾝを入力に設定
            //-------------------------------------------------
            if (Pasw.GetDirections(wLogSlot, ref dwDirections) == false)
            {
                ErrorMessage(wLogSlot);
                this.Close();
                return;
            }
            dwDirections = Pas500w.DIR_A_INPUT | dwDirections;  // A Port のﾋﾞｯﾄを入力に変更します。
            if (Pasw.SetDirections(wLogSlot, dwDirections) == false)
            {
                ErrorMessage(wLogSlot);
                this.Close();
                return;
            }
            ResultMessage("Port A のディレクションを入力に変更");
            //-------------------------------------------------
            // Port A からデータを読み込む
            //-------------------------------------------------
            cbDat = 0x0;
            if (Pasw.InPort(wLogSlot, Pas500w.PORT_A, ref cbDat) == false)
            {
                ErrorMessage(wLogSlot);
                this.Close();
                return;
            }
            szBuf = "Port A からデータを入力" + "\n" + "Port A In Data: &H" + Convert.ToString(cbDat, 16).ToUpper();
            ResultMessage(szBuf);
            //-------------------------------------------------
            // PA-S500の使用を開放
            //-------------------------------------------------
            if (Pasw.Close(wLogSlot) == false)
            {
                ErrorMessage(wLogSlot);
                this.Close();
                return;
            }
            ResultMessage("PA-S500 Closed");
            this.Close();
        }
        //============================================================================
        //    処理結果を表示
        //============================================================================
        private void ResultMessage(string lpszbuf)
        {
            MessageBox.Show(lpszbuf, this.Text, MessageBoxButtons.OK, MessageBoxIcon.Information);
        }
        //============================================================================
        //    エラーメッセージを表示
        //============================================================================
        private void ErrorMessage(short wLogSlot)
        {
            long dwRes = 0;
            string szBuf = null;

            dwRes = Pasw.GetLastError(wLogSlot);
            switch (dwRes)
            {
                case Pas500w.ERR_SYSTEM:
                    dwRes = System.Runtime.InteropServices.Marshal.GetLastWin32Error();
                    szBuf = string.Concat("System error", Environment.NewLine, Environment.NewLine, string.Format("Error code: &H{0:X8}", dwRes));
                    break;
                default:
                    szBuf = string.Concat("PA-S500 error", Environment.NewLine, Environment.NewLine, string.Format("Error code: &H{0:X8}", dwRes));
                    break;
            }
            MessageBox.Show(szBuf, this.Text, MessageBoxButtons.OK, MessageBoxIcon.Error);
        }

    }
}
